#-------------------------------------------------------------------------------
# Name:        module1
# Purpose:
#
# Author:      Jonas
#
# Created:     13/03/2012
# Copyright:   (c) Jonas 2012
# Licence:     <your licence>
#-------------------------------------------------------------------------------
#!/usr/bin/env python

import csnd
import scipy
import numpy
import math
from Enums import *
from scikits.audiolab import *
from pylab import *


def applyFlanger(inputFile, outputFile, flangerDelay):
    csdFile = "csd/flangerTest.csd"
    csound = csnd.Csound()                  # Csound instance
    compiled = csound.Compile(csdFile)
    if compiled != 0:
        print "Failed to compile ",csdFile
        return False
    csound.SetChannel("flanger", flangerDelay)
    csound.SetChannel("filename", inputFile)
    csound.Perform()
    copySuccess = createCopy(outputFile)
    if copySuccess:
        return True
    else:
        return False

def applyDistortion(inputFile, outputFile, distortionAmount):
    csdFile = "csd/distortTest.csd"
    csound = csnd.Csound()
    compiled = csound.Compile(csdFile)
    if compiled != 0:
        print "Failed to compile ",csdFile," with inputfile ",inputFile," and value ",distortionAmount
        return False
    csound.SetChannel("distort", distortionAmount)
    csound.SetChannel("filename", inputFile)
    csound.Perform()
    copySuccess = createCopy(outputFile)
    if copySuccess:
        return True
    else:
        return False

def applyTremolo(inputFile, outputFile, tremoloRate, tremoloDepth):
    csdFile = "csd/tremoloTest.csd"
    csound = csnd.Csound()
    compiled = csound.Compile(csdFile)
    if compiled != 0:
        print "Failed to compile ",csdFile," with inputfile ",inputFile," and value ",tremoloRate, ",",tremoloDepth
        return False
    csound.SetChannel("tremoloRate", tremoloRate)
    csound.SetChannel("tremoloDepth", tremoloDepth)
    csound.SetChannel("filename", inputFile)
    csound.Perform()
    copySuccess = createCopy(outputFile)
    if copySuccess:
        return True
    else:
        return False

def applyChorus(inputFile, outputFile, chorusRate, chorusDepth):
    csdFile = "csd/chorusTest.csd"
    csound = csnd.Csound()
    compiled = csound.Compile(csdFile)
    if compiled != 0:
        print "Failed to compile ",csdFile," with inputfile ",inputFile," and value ",chorusRate, ",",chorusDepth
        return False
    csound.SetChannel("chorusRate", chorusRate)
    csound.SetChannel("chorusDepth", chorusDepth)
    csound.SetChannel("filename", inputFile)
    csound.Perform()
    copySuccess = createCopy(outputFile)
    if copySuccess:
        return True
    else:
        return False

def applyReverb(inputFile):
    csdFile = "csd/freeverbTest.csd"
    csound = csnd.Csound()
    compiled = csound.Compile(csdFile)
    if compiled != 0:
        print "Failed to compile ",csdFile," with inputfile ",inputFile
        return False
    csound.SetChannel("reverb", 0.3)            # Setting 0.3 reverbdelay as default
    csound.SetChannel("filename", inputFile)
    csound.Perform()
    copySuccess = createCopy("sound.wav")
    if copySuccess:
        return True
    else:
        return False

def createCopy(inputFile):
    csdFile = "csd/createCopy.csd"
    csound = csnd.Csound()
    compiled = csound.Compile(csdFile)
    if compiled != 0:
        print "Failed to compile ",csdFile
        return False
    csound.SetChannel("filename", inputFile)
    csound.Perform()
    return True

# Creates a sound with no effects applied
def applyZero(inputFile):
    csdFile = "csd/zero.csd"
    csound = csnd.Csound()
    compiled = csound.Compile(csdFile)
    if compiled != 0:
        print "Failed to compile ",csdFile
        return False
    csound.SetChannel("filename", inputFile)
    csound.Perform()
    return True

# Used to create target sounds, and test sounds
def createSound(parameters, order, outputFile):
    if sum(parameters) == 0:
        applyZero("short.wav")
        return
    tempParameters = list(parameters)
    firstApplied = False
    for i in range(len(order)):
        if order[i] == Enums.DISTORTION:
            if tempParameters[0] < 1.0:
                del tempParameters[0]
                success = True
                continue
            if firstApplied:
                success = applyDistortion("copy.wav", outputFile, tempParameters[0])
                del tempParameters[0]
                continue
            else:
                success = applyDistortion("ctoneshort.wav", outputFile, tempParameters[0])
                del tempParameters[0]
                firstApplied = True
                continue
        if order[i] == Enums.FLANGER:
            if tempParameters[0] < 0:
                del tempParameters[0]
                success = True
                continue
            if firstApplied:
                success = applyFlanger("copy.wav", outputFile, tempParameters[0])
                del tempParameters[0]
                continue
            else:
                success = applyFlanger("ctoneshort.wav", outputFile, tempParameters[0])
                del tempParameters[0]
                firstApplied = True
                continue
        if order[i] == Enums.TREMOLO:
            if tempParameters[0] < 0 and tempParameters[1] < 0:
                del tempParameters[0]
                del tempParameters[0]
                success = True
                continue
            if tempParameters[0] < 0:
                tempParameters[0] = 0
            if tempParameters[1] < 0:
                tempParameters[1] = 0
            if firstApplied:
                success = applyTremolo("copy.wav", outputFile, tempParameters[0], tempParameters[1])
                del tempParameters[0]
                del tempParameters[0]
                continue
            else:
                success = applyTremolo("ctoneshort.wav", outputFile, tempParameters[0], tempParameters[1])
                del tempParameters[0]
                del tempParameters[0]
                firstApplied = True
                continue
        if order[i] == Enums.CHORUS:
            if tempParameters[0] < 0 and tempParameters[1] < 0:
                del tempParameters[0]
                del tempParameters[0]
                success = True
                continue
            if tempParameters[0] < 0:
                tempParameters[0] = 0
            if tempParameters[1] < 0:
                tempParameters[1] = 0
            if firstApplied:
                success = applyChorus("copy.wav", outputFile, tempParameters[0], tempParameters[1])
                del tempParameters[0]
                del tempParameters[0]
                continue
            else:
                success = applyChorus("ctoneshort.wav", outputFile, tempParameters[0], tempParameters[1])
                del tempParameters[0]
                del tempParameters[0]
                firstApplied = True

def plotSpecgram(filepath1, filepath2, fftLength, fs):
    sound_file1 = Sndfile(filepath1, 'r')
    signal1 = sound_file1.read_frames(sound_file1.nframes)
    sound_file2 = Sndfile(filepath2, 'r')
    signal2 = sound_file2.read_frames(sound_file2.nframes)
    figure(1)
    specgram(signal1, fftLength, fs)
    figure(2)
    specgram(signal2, fftLength, fs)
    show()

def hammingWindow(signal, N):
    for i in range(len(signal)):
        factor = 0.54 - (0.46 * math.cos((2*math.pi*i) / (N-1)))
        signal[i] = signal[i] * factor
    return signal

# Regular MSE distance emtric
def getDistance(filepath1, filepath2, fftLength):
    dist = 0.0
    sound_file1 = Sndfile(filepath1, 'r')
    sound_file2 = Sndfile(filepath2, 'r')
    if sound_file1.nframes == 0 or sound_file2.nframes == 0:
        getDistance(filepath1, filepath2, fftLength)
        return
    signal1 = sound_file1.read_frames(fftLength)
    signal2 = sound_file2.read_frames(fftLength)

    fft1 = abs(numpy.fft.fft(signal1))
    fft2 = abs(numpy.fft.fft(signal2))

    if len(fft1) == len(fft2):
        for i in range(len(fft1)):
            dist += math.pow((fft1[i] - fft2[i]), 2)
        dist = math.sqrt(dist)

    return dist

# Mitchells relative MSE distance metric
def relativeDistance(filepath1, filepath2, fftLength, numOfFrames):
    dist = 0.0
    sound_file1 = Sndfile(filepath1, 'r')
    sound_file2 = Sndfile(filepath2, 'r')
    if sound_file1.nframes == 0 or sound_file2.nframes == 0:
        relativeDistance(filepath1, filepath2, fftLength, numOfFrames)
        return
    signal1 = sound_file1.read_frames(sound_file1.nframes)
    signal2 = sound_file2.read_frames(sound_file2.nframes)
    step = int(sound_file1.nframes / numOfFrames)
    frames1 = []
    frames2 = []
    # Split signal into frames
    for i in range(0, step*numOfFrames, step):
        frame1 = signal1[i:i+step]
        frame2 = signal2[i:i+step]
        frames1.append(frame1)
        frames2.append(frame2)

    # Calculate relative MSE distance of the frames
    for i in range(len(frames1)):
        fft1 = abs(numpy.fft.fft(frames1[i]))
        fft2 = abs(numpy.fft.fft(frames2[i]))
        if len(fft1) == len(fft2):
            d = 0.0
            e = 0.0
            for j in range(len(fft1)):
                d += math.pow((fft1[j] - fft2[j]), 2)
                e += math.pow(fft2[j], 2)
            dist += math.sqrt(d/e)
    dist = dist / numOfFrames
    return dist
