from selectionmechanism import *
from selectionprotocol import *
from genetic_operators import *
from init import Init
from matplotlib import pyplot
import time


class GuitarProcessor:

    def __init__(self, numOfSounds, maxGenerations, crossoverType, mutationType, mechanism, protocol, cloneRate, mutationRate, orderMutation, orderMutationRate):
        self.numOfSounds = numOfSounds
        self.maxGenerations = maxGenerations
        self.crossoverType = crossoverType
        self.mutationType = mutationType
        self.mechanism = mechanism
        self.protocol = protocol
        self.cloneRate = cloneRate
        self.mutationRate = mutationRate
        self.orderMutation = orderMutation
        self.orderMutationRate = orderMutationRate

        self.targetFilepath = "csd/target.wav"
        self.targetInfo = "\nTarget: [0.031, 3.1, -0.1, -0.1, -0.1, -0.1], Order: Flanger, Distortion, Chorus, Tremolo"
        self.fftlength = 1024

    def run(self, Plot):
        timestamp = time.time()
        adults = []
        parents = []

        bestSound = Phenotype()

        # Plotting data
        fitness = []
        avgFitness = []
        title = ""
        pyplot.subplot(211)

        if self.mechanism == 1:
            title += "Fitness Proportionate - "

        if self.mechanism == 2:
            title += "Sigmascaling - "

        if self.mechanism == 3:
            title += "Tournament Selection - "

        if self.mechanism == 4:
            title += "Boltzmann Selection - "

        if self.protocol == 1:
            title += "Generational Replacement"

        if self.protocol == 2:
            title += "Overproduction"

        if self.protocol == 3:
            title += "Generational Mixing"

        title += "\nSounds: "
        title += str(self.numOfSounds)
        title += ", Mutation rate: "
        title += str(self.mutationRate)
        title += "\Clone rate: "
        title += str(self.cloneRate)
        title += "\nOrderMutation: "
        title += str(self.orderMutation)
        title += "\nOrderMutationRate: "
        title += str(self.orderMutationRate)



        init = Init(self.numOfSounds)

        if Plot:
            print self.numOfSounds," sounds created initially.."

        generation = 0

        # The evolution loop, exits when maxGenerations is reached or a perfect match is found
        while generation < self.maxGenerations and bestSound.fitness > 0.001:

            generation += 1
            print "Generation ",generation

            # Transform genotypes into phenotypes
            phenotypes = genoToPheno(init.sounds)

            # Test sounds against target sound and assign fitness
            testSounds(phenotypes, self.targetFilepath, self.fftlength)

            # Deciding protocol
            if self.protocol == 1:
                adults = generationalReplacement(phenotypes)

            if self.protocol == 2:
                adults = overProduction(phenotypes, adults)

            if self.protocol == 3:
                adults = generationalMixing(phenotypes, adults, self.numOfSounds)

            # Get best sound and save the result (lower fitness is better)
            bestSound = getBestSound(adults)
            fitness.append(bestSound.fitness)
            if Plot:
                avg = 0.0
                for i in range(len(adults)):
                    avg += adults[i].fitness
                avg = avg / len(adults)
                avgFitness.append(avg)
                print "\nAverage fitness: ",avg
                bestSound.printInfo()

            # Deciding mechanism
            if self.mechanism == 1:
                probtable = probTable(adults)
                parents = fitnessProportionate(adults, probtable, len(adults))

            if self.mechanism == 2:
                parents = sigmaScaling(adults, len(adults))

            if self.mechanism == 3:
                parents = tournamentSelection(adults, 3, 0.8)

            if self.mechanism == 4:
                parents = boltzmannSelection(adults, 5, len(adults))

            # Finally mating the chosen parents, to create new genotypes
            # using user determined rates and types of mutation and crossover

            init.sounds = mate(parents, self.crossoverType, self.cloneRate,
                                                        self.mutationType, self.mutationRate, self.orderMutation, self.orderMutationRate)

        #Plotting
        title += str(self.targetInfo)
        title += "\nBest: "
        title += str(bestSound.parameters)
        title += ", "
        title += bestSound.printOrder()
        pyplot.title(title)
        xscale = []

        for i in range(0, generation):
            xscale.append(i)

        # Graph
        if Plot:
            pyplot.ylim(0, max(avgFitness)+1)
            pyplot.plot(xscale, fitness, 'green', label='Best fitness')
            pyplot.plot(xscale, avgFitness, 'blue', label='Avg fitness')
            pyplot.legend()
            pyplot.xlabel('Generation')
            pyplot.ylabel('Fitness')

        bestSound.printInfo()
        print "\nTime used: ",(time.time()-timestamp) / 60.0," minutes"
        if Plot:
            pyplot.show()

        # Make sure the "sound.wav" is the best individual at the end (for listening test)
        testSound(bestSound, self.targetFilepath, self.fftlength)

        return bestSound
